;nyquist plug-in
;version 4
;type analyze
;name "Find Peaks"
;;debugbutton false
;debugflags trace
;author "Steve Daulton"
;release 2.4.2
;copyright "Licensed under terms of the GNU General Public License version 2"

;; Labels peaks that are within 'sensitivity (dB)' range below maximum peak.
;; When sensitivity = 0, only the absolute maximum peaks are labelled (usually only one).
;; Processing will stop if more than 1000 peaks are found and return labels
;; for those peaks only.


;control text "A small dB range will label the highest peaks only."
;control text "Increasing the dB range will label more (lesser) peaks."
;control text "Small label length is more precise, but slower."
;control threshold "dB Range" float "" 1 0 3
;control time-precision "Label length" float "ms" 1 0 100

(setf *max-labels* 1000)

(setf time-precision  (/ time-precision 1000.0))  ;seconds
(setf *step* (truncate (* time-precision *sound-srate*))) ;samples

;; Convert 'threshold' to absolute level (linear)
(let ((peak (get '*selection* 'peak-level)))
  (setf peak (linear-to-db peak))
  (setf threshold (db-to-linear (- peak threshold))))

;; Number of decimal places to display in label
(setf *float-format* "%.3f")


(defun monosig (sig step)
  ;;; Return mono sound of absolute sample values.
  ;;; When 'step' > 0, the sound is resampled to srate/step.
  (setf sig (s-abs sig))
  ; set *track*  to nil to avoid holding onto samples in ram.
  (setf *track* nil)
  (when (arrayp sig)
    (setf sig (s-max (aref sig 0)(aref sig 1))))
  (if (> step 0)
      (snd-avg sig step step op-peak)
      sig))


(defun get-peaks (sig thresh)
  ; Returns list of labels for peaks > thresh.
  ; Note: All sample values are absolute values.
  (let ((srate (snd-srate sig))
        (labels ())
        (label-count 0)
        (start-flag nil)
        (prev 0))
    (do ((val (snd-fetch sig) (snd-fetch sig))
         (count 0 (1+ count)))
        ((or (not val) (> label-count *max-labels*))
         (return-result labels label-count))
      (cond
        ((>= val thresh)
            (unless start-flag
              (setf start-flag count))
            (setf prev (max prev val)))
        (start-flag
            (push (make-label start-flag count srate prev)
                  labels)
            (incf label-count)
            (setf start-flag nil)
            (setf prev 0))))))


(defun make-label (start end srate val)
  ;;; Returns one label around 'count' position.
  (let ((start (/ start srate))
        (end (/ end srate))
        (db (linear-to-db val)))
    (list start end (format nil "~a dB" db))))


(defun return-result (labels label-count)
  ;; Handle return data
  (when (> label-count *max-labels*)
    (format t "Only the first ~a peaks will be labelled."
            *max-labels*))
  (if labels
      labels
      "Error.\nSelection too short for current settings."))


(get-peaks (monosig *track* *step*) threshold)
